import { default as React } from 'react';
import { ColumnKey } from './Column';
import { RowData, RowDefinition } from './Row';
import { CellDefinition } from './Cell';
import { SmartableData } from './Smartable';
import { Modify, Promisable } from '@kernelui/core';

/**
 * Current Smartable data state.
 */
export interface CurrentTableData<CK extends ColumnKey> {
    /**
     * Current rows state, undefined if they are still loading.
     */
    rows?: CurrentRowData<CK>[];
}
/**
 * Smartable current row cells data state.
 */
export type CurrentRowData<CK extends ColumnKey, T = any> = Modify<RowData<CK>, {
    cells: Record<CK, CellDefinition<T>>;
}>;
/**
 * Get current async table data manager.
 */
export declare function useAsyncManager<CK extends ColumnKey>(data: SmartableData<CK>): AsyncManager<CK>;
/**
 * Smartable async data manager.
 */
declare class AsyncManager<CK extends ColumnKey> {
    currentDataState: CurrentTableData<CK>;
    setCurrentDataState: React.Dispatch<React.SetStateAction<CurrentTableData<CK>>>;
    /**
     * Main promised data.
     * @protected
     */
    protected promisedData: Promised<SmartableData<CK>>;
    /**
     * Promised rows data.
     * @protected
     */
    protected promisedRows: Promised<RowDefinition<CK>>[];
    /**
     * Promised rows cells data.
     * @protected
     */
    protected promisedRowsCells: Partial<Record<CK, Promised<CellDefinition>>>[];
    /**
     * Tells if rows are loaded or not.
     * @protected
     */
    protected rowsLoaded: boolean;
    /**
     * Tells if rows need to be reinitialized or not.
     * @protected
     */
    protected reinitRows: boolean;
    /**
     * Rows data.
     * @protected
     */
    protected rowsData: RowData<CK>[];
    /**
     * Rows cells definitions.
     * @protected
     */
    protected cellsDefinitions: Partial<Record<CK, CellDefinition<CK>>>[];
    constructor();
    /**
     * Handle new smartable data.
     * @param data Smartable data.
     */
    handle(data: SmartableData<CK>): void;
    /**
     * Handle new rows definitions.
     * @param rowsDefinitions Rows definitions.
     * @protected
     */
    protected handleNewRowsDefinitions(rowsDefinitions: Promisable<RowDefinition<CK>>[]): void;
    /**
     * Handle a new row.
     * @param rowId Row ID.
     * @param row Row definition.
     * @protected
     */
    protected handleNewRow(rowId: number, row: RowDefinition<CK>): void;
    /**
     * Handle a new cell.
     * @param rowId Cell row ID.
     * @param columnKey Cell column key.
     * @param cellDefinition Cell definition.
     * @protected
     */
    protected handleNewCell(rowId: number, columnKey: CK, cellDefinition: CellDefinition): void;
    /**
     * True if there is a pending update.
     * @protected
     */
    protected pendingUpdate: boolean;
    /**
     * Try to update the current data state, if there are no more changes in the next 25ms.
     * The next update will be in max 25ms.
     * @protected
     */
    protected tryToUpdate(): void;
    /**
     * Update the current data state with the loaded rows and cells data.
     * @protected
     */
    protected update(): void;
}
/**
 * Promised data class.
 */
export declare class Promised<T> {
    /**
     * The main data promise.
     */
    promise?: Promise<T>;
    /**
     * Data retrieved from promise or given in parameter.
     */
    data?: T;
    /**
     * Called when data is changed.
     */
    onData?: (data: T) => void;
    constructor(onChanged?: (data: T) => void, data?: Promisable<T>);
    /**
     * Refresh the promised data.
     * @param data Promised data.
     */
    refresh(data: Promisable<T>): this;
    /**
     * Return true if some data (or its promise) have been provided.
     */
    isInitialized(): boolean;
    /**
     * Return true if we are waiting for a promise result.
     */
    isLoading(): boolean;
}
export {};
