import { default as React } from 'react';
import { Smartable } from './Smartable';
import { ColumnFilter } from './Columns/ColumnFilter';
import { SortState } from './Sort';

/**
 * Basic column key type.
 */
export type ColumnKey = string | symbol;
/**
 * Smartable columns definition.
 */
export type Columns<K extends ColumnKey> = Record<K, Column>;
/**
 * Define columns for a Smartable.
 * @param columns
 */
export declare function createColumns<K extends ColumnKey>(...columns: [K, Column][]): Columns<K>;
/**
 * Smartable column definition.
 */
export interface Column<T = any> {
    /**
     * Column title element.
     */
    title: React.ReactNode;
    /**
     * Column cell default element.
     */
    cellElement?: React.ReactElement;
    /**
     * Cells data comparison in the column.
     * @param a First data to compare.
     * @param b Second data to compare.
     */
    compare?: (a: T, b: T) => number;
    /**
     * Column filter definition.
     */
    filter?: ColumnFilter;
}
/**
 * Create a column for a Smartable.
 * @param key The column key, which should be unique per Smartable instance.
 * @param column The column definition.
 */
export declare function createColumn<K extends ColumnKey>(key: K, column: Column): [K, Column];
/**
 * Table column context data.
 */
export interface ColumnContextData<CK extends ColumnKey> {
    /**
     * Column key.
     */
    key: ColumnKey;
    /**
     * Column definition.
     */
    column: Column;
    /**
     * Column sort state.
     */
    sortState?: SortState;
    /**
     * Column filter state.
     */
    filterState: any;
    /**
     * Set current column filter state.
     * @param filterState New filter state.
     */
    setFilterState: <T = any>(filterState: T) => void;
}
export declare const ColumnContext: React.Context<ColumnContextData<ColumnKey>>;
/**
 * Hook to get current column data.
 */
export declare function useColumn<CK extends ColumnKey>(smartable?: Smartable<CK>): ColumnContextData<CK>;
/**
 * Auto column context provider from table context.
 */
export declare function AutoColumnContextProvider({ columnKey, children }: React.PropsWithChildren<{
    columnKey: ColumnKey;
}>): React.JSX.Element;
