import { default as React } from 'react';
import { ColumnKey, Columns } from './Column';
import { RowDefinition } from './Row';
import { Promisable } from '@kernelui/core';
import { SortState, SortType } from './Sort';

/**
 * Smartable data type.
 */
export type SmartableData<CK extends ColumnKey> = Promisable<(Promisable<RowDefinition<CK>>)[]>;
/**
 * Smartable properties.
 */
export interface SmartableProperties<CK extends ColumnKey> {
    /**
     * Table custom class name.
     */
    className?: string;
    /**
     * Table data.
     */
    data: SmartableData<CK>;
    /**
     * When defined, only show the columns in the array.
     */
    shownColumns?: CK[];
    /**
     * Default row element.
     */
    rowElement?: React.ReactElement;
    /**
     * Default column heading element.
     */
    columnHeadingElement?: React.ReactElement;
    /**
     * Table rows pagination.
     */
    paginate?: {
        /**
         * Number of rows per page.
         */
        pageSize: number;
    };
    /**
     * Disable sort feature.
     */
    disableSort?: boolean;
    /**
     * Disable filter feature.
     */
    disableFilter?: boolean;
}
/**
 * Smartable main object.
 */
export type Smartable<CK extends ColumnKey> = {
    /**
     * Table component.
     */
    Table: React.FunctionComponent<SmartableProperties<CK>>;
};
/**
 * Smartable columns infered type.
 */
export type SmartableColumns<CK> = CK extends Smartable<infer CK> ? CK : never;
/**
 * Define a new Smartable.
 */
export declare function createSmartable<CK extends ColumnKey>({ columns }: {
    /**
     * Smartable table columns.
     */
    columns: Columns<CK>;
}): Smartable<CK>;
/**
 * Smartable table context data.
 */
export interface TableContextData<CK extends ColumnKey> extends SmartableProperties<CK> {
    /**
     * Current table columns.
     */
    columns: Columns<CK>;
    /**
     * Current table columns sort state.
     */
    columnsSortState: Partial<Record<CK, SortState>>;
    /**
     * Set given table column sort state.
     * @param key The column key for which to set the sort type.
     * @param sortType The sort type to set for the given column. NULL to reset sort state.
     */
    setColumnSortState: (key: CK, sortType: SortType | null) => void;
    /**
     * Current table columsn filter states.
     */
    columnsFilterStates: Partial<Record<CK, any>>;
    /**
     * Set given table column filter state.
     * @param key The column key for which to set the filter state.
     * @param filterState The filter state to set for the given column.
     */
    setColumnFilterState: (key: CK, filterState: any) => void;
}
/**
 * Hook to get current Smartable table context.
 * @param smartable The Smartable main object.
 */
export declare function useTable<CK extends ColumnKey>(smartable?: Smartable<CK>): TableContextData<CK>;
